# Skript:
# -------
# make-crypt.sh
# 
# zugehriges Skript:
# make-crypt2.sh
# 
# Aufruf nur als User root:
# -------------------------
# ./make-crypt.sh
#
# ============================================================================
# Author:  Friedhelm Papenburg 
# Version: 2002.05.20
#
# Copyright (C) 2002  Friedhelm Papenburg.
# Die Scripte unterliegen dem GNU Copyright: 
# GNU GENERAL PUBLIC LICENCE Version 2, June 1991
# der Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# Die Aenderungshistorie darf erweitert werden.
# Die bestehenden Daten der Aenderungshistorie duerfen nicht geaendert werden.
# Aenderungen muessen in der Aenderungshistorie protokolliert werden.
#
# Aenderungshistorie:
#
# Datum      Author                    Aenderungsgrund
# ========== ========================= =======================================
# 2002.05.20 Friedhelm Papenburg       Ersterstellung
# xxxx.xx.xx yyyyyyyyyyyyyyyyyyyyyyyyy zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz
#
# ============================================================================
#
# Funktion:
# ---------
# Verschluesseltes Verzeichnisnis /crypthome/test/crypttest anlegen
# (hier mit 500 MB) fuer User test.
# Bitte im Skript berall test zu Ihrem gewuenschten Usernamen aendern !!!
# Vorgegeben ist der User test. Der User muss existieren.
#
#
# Voraussetzung:
# --------------
# 1) Rootrechte
# 2) Es existiert ein Benutzer test
# 3) Suse Linux 7.3 (allgemeiner: eine Linuxversion, dessen Kernel die
#    Twofish Verschluesselung unterstuetzt)
# 4) Zunaechst sollten Sie einen User test anlegen und damit erstmal alles
#    probieren!!!
#
#
# Vorgehensweise:
# ---------------
# Einloggen als User root
# In einem Text-Editor in den beiden Scripten berall test durch
# Ihren gewnschten Usernamen ersetzen (ohne diese Aenderung 
# funktionieren die Skripte fuer den User test).
# Aendern der Dateien fstab und boot.local wie unten beschrieben.
# In fstab ebenfalls statt test den gewuenschten Usernamen waehlen.
# Aufruf des Skriptes mittels:
# ./make-crypt.sh
# merken der Passphrase und dann
# neu Booten.
# Nun zwei Mglichkeiten:
# 1) Als User test (bzw. Ihren gewaehlten Usernamen) einloggen
#    und mittels:
#    mount /crypthome/test/crypttest
#    und Eingabe der Passphrase 
#    arbeiten mit dem Verzeichnis /crypthome/test/crypttest
# 2) Als User root einloggen und einmalig
#    ./make-crypt2.sh 
#    aufrufen, dann
#    neu booten und als User test einloggen, dabei
#    Userpasswort und Passphrase eingeben.
#    Es wird ab dann immer mit dem verschluesseltem Verzeichnis
#    /home/test gearbeitet.  
#
#
# Eingabe fuer make-crypt.sh:
# ---------------------------
# Das Programm losetup verlangt die Eingabe einer Passphrase,
# das ist ein aus Sicherheitsgruenden sehr langes Passwort,
# welches Sie frei waehlen. 
# Diese Passphrase muessen Sie immer beim Mounten des verschluesselten
# Verzeichnisses eingeben 
#
#
# Vorarbeiten die erledigt werden muessen:
# ----------------------------------------
# In Datei boot.local am Ende eintragen:
# modprobe loop_fish2
#
# In Datei fstab eintragen:
# /crypthome/test/.crypttest /crypthome/test/crypttest ext2 defaults,noauto,loop,encryption=twofish,user 0 0
# 
#
# Mounten des verschluesselten Verzeichnisses:
# --------------------------------------------
# 1) Sie haben nicht ./make-crypt2.sh aufgerufen: 
#    nach jedem neuen Bootvorgang nuessen Sie das verschluesselte Verzeichnis
#    neu mounten, wobei die nicht mehr aenderbare Passphrase jedesmal
#    eingegeben werden muss.
#    Als User test geben Sie dazu ein:
#    mount /crypthome/test/crypttest
#
#    mittels:
#    umount /crypthome/test/crypttest
#    ist das Verzeichnis dem Zugriff wieder entzogen und sicher.
#
# 2) Sie haben ./make-crypt2.sh aufgerufen:
#    Das Verzeichnis /home/test ist verschluesselt und wird automatisch
#    Beim Einloggen gemountet.
#
#    mittels:
#    cd /home
#    umount /crypthome/test/crypttest
#    ist das Verzeichnis dem Zugriff wieder entzogen und sicher.
#
#
# Achtung:
# --------
# 1) Das Skript make-crypt.sh duerfen Sie nur einmal als User root
# aufrufen. Vor dem neuen Booten muessen die Dateien fstab
# und boot.local wie oben beschrieben angepasst werden.
# Nach jedem neuen Bootvorgang muss der User test den Befehl
# mount /crypthome/test/crypttest unter Eingabe der Passphrase eingeben.
# 2) In der Regel befindet sich die Datei fstab im Verzeichnis /etc
# Die Datei boot.local finden Sie mit den Befehlen:
# cd /
# find . -name boot.local
# 3) Achten Sie unbedingt darauf, in dem Verzeichniss fuer die 
# verschluesselte Datei genuegend Platz zu haben.
# Den freien Platz sehen Sie z.B. mittels des Befehls:
# df -h 
#
#
# Sicherheit:
# -----------
# 1) Der Twofisch Algorythmus ist nach heutigem Wissensstand sehr sicher.
# 2) Eine Schwachstelle bildet - wie bei jedem Algorythmus - das Passwort. 
#    Ein langes Passwort auch Passphrase genannt mit Sonderzeichen 
#    ist sehr sicher.
# 3) Bei der hier beschrieben Verschluesselung und Verfahrensweise kann
#    nur der Superuser und alle denen Sie Zugriffsrechte gewaehren,
#    zu dem Zeitpunkt wo Ihr verschluesseltes Verzeichniss gemountet ist,
#    auf Ihre Daten zugreifen. Ist Ihr Verzeichnis nicht - mit Hilfe der
#    hoffentlich nur Ihnen bekannten Passphrase - gemountet, so kann
#    keiner mit vertretbarem Aufwand Ihre Daten entschluesseln.
#    Wenn also jemand Ihren Rechner mitnimmt und sogar das Rootpasswort 
#    kennt, aber nicht Ihre Passphrase, so kann er nichts mit Ihrem
#    verschluesseltem Verzeichniss anfangen. 
#      
#
# Moegliche Anpassungen des Skripts:
# ----------------------------------
# Statt des Users test koenen Sie auch einen anderen existierenden
# Usernamen waehlen.
# Auch fuer das Verzeichnis crypthome koenen Sie einen anderen Namen waehlen.
# Auch die Groesse des verschluesselten Verzeichnisses koennen Sie
# anpassen: Parameter count im dd-Befehl. Z.B. bei count = 500 werden
# 500 MB reserviert.
# Die anzupassende Zeilen ist die mit: UGROESSE=500
# Fuer eine Verschluesselung des gesamten Verzeichnisse /home/test
# bitte die Zeilen hinter dem exit Befehl beachten(siehe auch make-crypt2.sh).
# Bitte auch in fstab den Usernamen test anpassen.
#
#
# Wiederherstellen des verschluesselten Verzeichnisses /home/test als
# unverschluesseltes Verzeichnis:
# -------------------------------------------------------------------
# als user root:
# cd /home
# mkdir test-ret
# chown test test-ret
# mount /crypthome/test/crypttest
# cd /crypthome/test/crypttest
# find . -print | cpio -pvumd /home/test-ret
# cd /crypthome/test
# umount /crypthome/test/crypttest
# rm * -r
# rm .* -r
# cd /home
# rm test
# mv test-ret test
# chown test test
#
# Anmerkung: wenn Sie die Passphrase vergessen haben,
#            sind Ihre Daten fuer immer verloren!
# 
# 
# ============================================================================
# nachfolgende Variable bitte an Ihre Wuensche anpassen
# UGROESSE ist Groesse des verschluesselten Verzeichnisses in MB, 
#          hier UGROESSE=500
UGROESSE=500
#
# ============================================================================
#
echo
echo Skript make-crypt.sh
echo Funktion: Anlegen eines verschluesselten Verzeichnisses
echo Zugehoeriges Skript: make-crypt2.sh
echo Voraussetzung: Linuxkernel, der die Twofish Verschluesselung unterstuetzt
echo Getestet mit SuSE 7.3.
echo
echo Achtung der User test muss existieren, sonst funkioniert das Skript nicht.
echo Es muessen mindestens $UGROESSE MB fuer das Verzeichniss /crypthome
echo zur Verfuegung stehen. 
echo Das verschluesselte Verzeichnis erhaelt eine Groesse von $UGROESSE MB.
echo
cd /
mkdir crypthome
cd /crypthome
mkdir test
cd test
#
# Verschlusselungsmodul fuer die Twofish-Verschluesselung laden.
modprobe loop_fish2
#
# Datei /crypthome/test/.crypttest anlegen, welche spaeter die
# verschluesselten Daten enthaelt. Das Verzeichnis /crypthome/test/crypttest
# wird (siehe weiter unten) in diese Datei gemountet.
echo
echo Bitte warten - rund 2 Minuten bei Rechner mit 1,6 GHz und 500 MB RAM - 
echo es wird die Datei fuer das verschluesselte Verzeichnis erzeugt.
if dd if=/dev/urandom of=/crypthome/test/.crypttest bs=1024k count=$UGROESSE
then
   echo 'Datei /crypthome/test.crypttest erfolgreich angelegt.'
else
   echo 'Fehler: wahrscheinlich nicht genug Platz in /crypthome.'
   exit 1
fi    
# /crypthome/test/.crypttest mit dem Loopback-Device verbinden.
# und die Passphrase eingeben und merken fuer den immer wieder
# einzugebenden mount Befehl.
echo Bitte Passphrase eingeben und merken  
losetup -e twofish /dev/loop1 /crypthome/test/.crypttest
# ext2 Dateisystem anlegen (in Datei /crypthome/test/.crpto) 
mke2fs /dev/loop1
mkdir /crypthome/test/crypttest
chown test /crypthome/test/crypttest
# 
mount -t ext2 /dev/loop1 /crypthome/test/crypttest
# die richtigen Zugriffsrechte setzen
chown test:root /crypthome/test/.crypttest
chmod go= /crypthome/test/.crypttest
chown test:root -R /crypthome/test/crypttest
chmod go= -R /crypthome/test/crypttest
# nun kann man die Verbindung wieder loesen
umount /dev/loop1
losetup -d /dev/loop1
#
echo
echo Achtung vor dem neuen Booten vergessen Sie bitte nicht die
echo Eintraege in fstab und boot.local.
echo in boot.local:
echo modprobe loop_fish2
echo in fstab:
echo /crypthome/test/.crypttest /crypthome/test/crypttest ext2 defaults,noauto,loop,encryption=twofish,user 0 0
echo 
#
echo Nun zwei Moeglichkeiten:
echo '1): Nach dem Booten mounten Sie als User test das verschluesselte'
echo '    Verzeichnis /crypthome/test/crypttest unter Eingabe der' 
echo '    Passphrase mittels:'
echo '    mount /crypthome/test/crypttest'
echo '2): Nach dem Booten als user root einmalig aufrufen:'
echo '    ./make-crypt2.sh'
echo '    Dadurch wird das bestehende Verzeichnis /home/test verschluesselt.'
echo '    Ab jetzt arbeiten Sie immer mit diesem verschluesseltem Verzeichnis'
echo '    indem Sie sich einfach als User test unter Eingabe des Passwortes'
echo '    und der Passphrase anmelden.'
echo '    Mittels:'
echo '    cd /home'
echo '    umount /crypthome/test/crypttest'
echo '    entziehen Sie Ihr verschluesseltes Verzeichniss wieder dem Zugriff.'
echo '    Durch Runterfahren wird das Verzeichnis ebenfalls unzugaenglich.'
#
#
exit
#
#
# ============================================================================
#
# Statt das nachfolgende per Hand einzugeben, koenne Sie auch als User root
# das Skript make-crypt2.sh aufrufen.
#
# Achtung nachfolgenden rm Befehl erst absetzen, wenn alles OK ist!!!
# Bitte fuer test Ihren Usernamen einsetzen!
# Will der User test fuer verschluesselte Dateien nicht mit dem
# Verzeichnis /crypthome/test/crypttest arbeiten, sondern sein eigenes
# Verzeichnis /home/test selbst verschluesseln, so kann er dies ebenfalls
# tun, indem er einmalig nach dem erneuten Hochfahren und einloggen als 
# User root folgende 9 Befehle absetzt:
UNAME=test
mount /crypthome/test/crypttest
cd /home/test
find . -print | cpio -pvumd /crypthome/test/crypttest
cd /home
rm /home/test/* -r
rm /home/test/.* -r
rmdir test
ln -s /crypthome/test/crypttest /home/test
chown test /home/test
#
# ab nun sind nur nach dem jeweiligen Einloggen als User test folgende
# Befehle abzusetzen:
UNAME=test
cd /home
mount /crypthome/test/crypttest
cd /home/test
#
# um sich auch das noch zu sparen kann der User root
# obige 4 Befehle in die Datei /home/test/.profile bringen
# Dabei muss aber zuvor dass Kommando:
umount /crypthome/test/crypttest
# eingegeben werden.
# danach nicht vergessen:
chown test /home/test
chown test /home/test/.profile
chmod u+x test /home/test/.profile
#
# dann muss nach dem Einloggen nur noch das Passwort fuer den User und
# anschliessend das Passwort fuer die Verschluesselung eingegeben werden.
#
# ============================================================================
















